@@ -20,6 +20,7 @@ from oslo_utils import uuidutils
 from six import moves
 
 from neutron.common import constants as n_const
+from neutron.common import ipv6_utils
 
 _uuid = uuidutils.generate_uuid
 
@@ -40,6 +41,7 @@ def get_ha_interface(ip='169.254.192.1', mac='12:34:56:78:2b:5d'):
             'id': _uuid(),
             'mac_address': mac,
             'name': u'L3 HA Admin port 0',
+            'mtu': 1500,
             'network_id': _uuid(),
             'status': u'ACTIVE',
             'subnets': [{'cidr': '169.254.192.0/18',
@@ -91,6 +93,7 @@ def prepare_router_data(ip_version=4, enable_snat=None, num_internal_ports=1,
     if enable_gw:
         ex_gw_port = {'id': _uuid(),
                       'mac_address': gateway_mac,
+                      'mtu': 1500,
                       'network_id': _uuid(),
                       'fixed_ips': fixed_ips,
                       'subnets': subnets,
@@ -181,6 +184,7 @@ def router_append_interface(router, count=1, ip_version=4, ra_mode=None,
 
         interfaces.append(
             {'id': _uuid(),
+             'mtu': 1500,
              'network_id': _uuid(),
              'admin_state_up': True,
              'fixed_ips': fixed_ips,
@@ -273,6 +277,7 @@ def router_append_pd_enabled_subnet(router, count=1):
     for i in range(current, current + count):
         subnet_id = _uuid()
         intf = {'id': _uuid(),
+                'mtu': 1500,
                 'network_id': _uuid(),
                 'admin_state_up': True,
                 'fixed_ips': [{'ip_address': '::1',
@@ -287,6 +292,26 @@ def router_append_pd_enabled_subnet(router, count=1):
         interfaces.append(intf)
         pd_intfs.append(intf)
         mac_address.value += 1
+
+
+def get_unassigned_pd_interfaces(router):
+    pd_intfs = []
+    for intf in router[lib_constants.INTERFACE_KEY]:
+        for subnet in intf['subnets']:
+            if (ipv6_utils.is_ipv6_pd_enabled(subnet) and
+                subnet['cidr'] == n_const.PROVISIONAL_IPV6_PD_PREFIX):
+                pd_intfs.append(intf)
+    return pd_intfs
+
+
+def assign_prefix_for_pd_interfaces(router):
+    pd_intfs = []
+    for ifno, intf in enumerate(router[lib_constants.INTERFACE_KEY]):
+        for subnet in intf['subnets']:
+            if (ipv6_utils.is_ipv6_pd_enabled(subnet) and
+                subnet['cidr'] == n_const.PROVISIONAL_IPV6_PD_PREFIX):
+                subnet['cidr'] = "2001:db8:%d::/64" % ifno
+                pd_intfs.append(intf)
     return pd_intfs
 
 
@@ -310,6 +335,7 @@ def prepare_ext_gw_test(context, ri, dual_stack=False):
                   'subnets': subnets,
                   'extra_subnets': [{'cidr': '172.16.0.0/24'}],
                   'id': _uuid(),
+                  'mtu': 1500,
                   'network_id': _uuid(),
                   'mac_address': 'ca:fe:de:ad:be:ef'}
     interface_name = ri.get_external_device_name(ex_gw_port['id'])
