@@ -490,9 +490,8 @@ class TestDvrRouter(framework.L3AgentTestFramework):
                 floating_ip['port_id'] = internal_ports[0]['id']
                 floating_ip['status'] = 'ACTIVE'
 
-            if not snat_bound_fip:
-                self._add_fip_agent_gw_port_info_to_router(router,
-                                                           external_gw_port)
+            self._add_fip_agent_gw_port_info_to_router(router,
+                                                       external_gw_port)
         return router
 
     def _get_fip_agent_gw_port_for_router(
@@ -1019,6 +1018,42 @@ class TestDvrRouter(framework.L3AgentTestFramework):
             self._assert_iptables_rules_exist(
                 router1.snat_iptables_manager, 'nat', expected_rules)
 
+    def test_floating_ip_migration_from_unbound_to_bound(self):
+        """Test to check floating ips migrate from unboun to bound host."""
+        self.agent.conf.agent_mode = lib_constants.L3_AGENT_MODE_DVR_SNAT
+        router_info = self.generate_dvr_router_info(
+            enable_floating_ip=True, enable_centralized_fip=True,
+            enable_snat=True, snat_bound_fip=True)
+        router1 = self.manage_router(self.agent, router_info)
+        centralized_floatingips = router_info[lib_constants.FLOATINGIP_KEY]
+        # For private ports hosted in dvr_no_fip agent, the floatingip
+        # dict will contain the fip['host'] key, but the value will always
+        # be None to emulate the legacy router.
+        self.assertIsNone(centralized_floatingips[0]['host'])
+        self.assertTrue(self._namespace_exists(router1.ns_name))
+        fip_ns = router1.fip_ns.get_name()
+        self.assertTrue(self._namespace_exists(fip_ns))
+        self._assert_snat_namespace_exists(router1)
+        # If fips are centralized then, the DNAT rules are only
+        # configured in the SNAT Namespace and not in the router-ns.
+        router_ns = router1.ns_name
+        fixed_ip = centralized_floatingips[0]['fixed_ip_address']
+        for fip in centralized_floatingips:
+            expected_rules = router1.floating_forward_rules(fip)
+            self.assertFalse(self._assert_iptables_rules_exist(
+                router1.snat_iptables_manager, 'nat', expected_rules))
+        self.assertFalse(self._fixed_ip_rule_exists(router_ns, fixed_ip))
+        # Now let us edit the floatingIP info with 'host' and remove
+        # the 'dvr_snat_bound'
+        router1.router[lib_constants.FLOATINGIP_KEY][0]['host'] = (
+            self.agent.conf.host)
+        del router1.router[lib_constants.FLOATINGIP_KEY][0]['dvr_snat_bound']
+        self.agent._process_updated_router(router1.router)
+        router_updated = self.agent.router_info[router_info['id']]
+        router_ns = router_updated.ns_name
+        self.assertTrue(self._fixed_ip_rule_exists(router_ns, fixed_ip))
+        self.assertTrue(self._namespace_exists(fip_ns))
+
     def test_floating_ip_not_deployed_on_dvr_no_external_agent(self):
         """Test to check floating ips not configured for dvr_no_external."""
         self.agent.conf.agent_mode = n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL
@@ -1089,11 +1124,16 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         self.assertFalse(sg_device)
         self.assertTrue(qg_device)
 
-    def _mocked_dvr_ha_router(self, agent, enable_gw=True):
-        r_info = self.generate_dvr_router_info(enable_ha=True,
-                                               enable_snat=True,
-                                               agent=agent,
-                                               enable_gw=enable_gw)
+    def _mocked_dvr_ha_router(self, agent, enable_ha=True, enable_gw=True,
+                              enable_centralized_fip=False,
+                              snat_bound_fip=False):
+        r_info = self.generate_dvr_router_info(
+            enable_ha=enable_ha,
+            enable_snat=True,
+            agent=agent,
+            enable_gw=enable_gw,
+            enable_centralized_fip=enable_centralized_fip,
+            snat_bound_fip=snat_bound_fip)
 
         r_snat_ns_name = namespaces.build_ns_name(dvr_snat_ns.SNAT_NS_PREFIX,
                                                   r_info['id'])
@@ -1122,19 +1162,75 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         br_int_1.add_port(veth1.name)
         br_int_2.add_port(veth2.name)
 
-    def _create_dvr_ha_router(self, agent, enable_gw=True):
+    def _create_dvr_ha_router(self, agent, enable_gw=True,
+                              enable_centralized_fip=False,
+                              snat_bound_fip=False, ha_interface=True):
         get_ns_name = mock.patch.object(namespaces.RouterNamespace,
                                         '_get_ns_name').start()
         get_snat_ns_name = mock.patch.object(dvr_snat_ns.SnatNamespace,
                                              'get_snat_ns_name').start()
         (r_info,
          mocked_r_ns_name,
-         mocked_r_snat_ns_name) = self._mocked_dvr_ha_router(agent, enable_gw)
+         mocked_r_snat_ns_name) = self._mocked_dvr_ha_router(
+             agent, ha_interface, enable_gw, enable_centralized_fip,
+             snat_bound_fip)
+
+        if not ha_interface:
+            r_info['ha'] = True
+
         get_ns_name.return_value = mocked_r_ns_name
         get_snat_ns_name.return_value = mocked_r_snat_ns_name
         router = self.manage_router(agent, r_info)
         return router
 
+    def _assert_ip_addresses_in_dvr_ha_snat_namespace_with_fip(self, router):
+        namespace = router.ha_namespace
+        ex_gw_port = router.get_ex_gw_port()
+        snat_ports = router.get_snat_interfaces()
+        if not snat_ports:
+            return
+        if router.is_router_master():
+            centralized_floatingips = (
+                router.router[lib_constants.FLOATINGIP_KEY])
+            for fip in centralized_floatingips:
+                expected_rules = router.floating_forward_rules(fip)
+                self.assertFalse(self._assert_iptables_rules_exist(
+                    router.snat_iptables_manager, 'nat', expected_rules))
+
+        snat_port = snat_ports[0]
+        ex_gw_port_name = router.get_external_device_name(
+            ex_gw_port['id'])
+        snat_port_name = router._get_snat_int_device_name(
+            snat_port['id'])
+
+        ex_gw_port_cidrs = utils.fixed_ip_cidrs(ex_gw_port["fixed_ips"])
+        snat_port_cidrs = utils.fixed_ip_cidrs(snat_port["fixed_ips"])
+
+        self._assert_ip_addresses_on_interface(namespace,
+                                               ex_gw_port_name,
+                                               ex_gw_port_cidrs)
+        self._assert_ip_addresses_on_interface(namespace,
+                                               snat_port_name,
+                                               snat_port_cidrs)
+
+    def _assert_no_ip_addresses_in_dvr_ha_snat_namespace_with_fip(self,
+                                                                  router):
+        namespace = router.ha_namespace
+        ex_gw_port = router.get_ex_gw_port()
+        snat_ports = router.get_snat_interfaces()
+        if not snat_ports:
+            return
+        snat_port = snat_ports[0]
+        ex_gw_port_name = router.get_external_device_name(
+            ex_gw_port['id'])
+        snat_port_name = router._get_snat_int_device_name(
+            snat_port['id'])
+
+        self._assert_no_ip_addresses_on_interface(namespace,
+                                                  snat_port_name)
+        self._assert_no_ip_addresses_on_interface(namespace,
+                                                  ex_gw_port_name)
+
     def _assert_ip_addresses_in_dvr_ha_snat_namespace(self, router):
         namespace = router.ha_namespace
         ex_gw_port = router.get_ex_gw_port()
@@ -1180,6 +1276,33 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         self._assert_no_ip_addresses_on_interface(namespace,
                                                   ex_gw_port_name)
 
+    def _test_dvr_ha_router_failover_with_gw_and_fip(self, enable_gw,
+                                                     enable_centralized_fip,
+                                                     snat_bound_fip):
+        self._setup_dvr_ha_agents()
+        self._setup_dvr_ha_bridges()
+
+        router1 = self._create_dvr_ha_router(
+            self.agent, enable_gw=enable_gw,
+            enable_centralized_fip=enable_centralized_fip,
+            snat_bound_fip=snat_bound_fip)
+        router2 = self._create_dvr_ha_router(
+            self.failover_agent, enable_gw=enable_gw,
+            enable_centralized_fip=enable_centralized_fip,
+            snat_bound_fip=snat_bound_fip)
+        utils.wait_until_true(lambda: router1.ha_state == 'master')
+        utils.wait_until_true(lambda: router2.ha_state == 'backup')
+
+        self._assert_ip_addresses_in_dvr_ha_snat_namespace_with_fip(router1)
+        self._assert_no_ip_addresses_in_dvr_ha_snat_namespace_with_fip(router2)
+        self.fail_ha_router(router1)
+
+        utils.wait_until_true(lambda: router2.ha_state == 'master')
+        utils.wait_until_true(lambda: router1.ha_state == 'backup')
+
+        self._assert_ip_addresses_in_dvr_ha_snat_namespace_with_fip(router2)
+        self._assert_no_ip_addresses_in_dvr_ha_snat_namespace_with_fip(router1)
+
     def _test_dvr_ha_router_failover(self, enable_gw):
         self._setup_dvr_ha_agents()
         self._setup_dvr_ha_bridges()
@@ -1204,9 +1327,33 @@ class TestDvrRouter(framework.L3AgentTestFramework):
     def test_dvr_ha_router_failover_with_gw(self):
         self._test_dvr_ha_router_failover(enable_gw=True)
 
+    def test_dvr_ha_router_failover_with_gw_and_floatingip(self):
+        self._test_dvr_ha_router_failover_with_gw_and_fip(
+            enable_gw=True, enable_centralized_fip=True, snat_bound_fip=True)
+
     def test_dvr_ha_router_failover_without_gw(self):
         self._test_dvr_ha_router_failover(enable_gw=False)
 
+    def test_dvr_non_ha_router_update(self):
+        self._setup_dvr_ha_agents()
+        self._setup_dvr_ha_bridges()
+
+        router1 = self._create_dvr_ha_router(self.agent)
+        router2 = self._create_dvr_ha_router(self.failover_agent,
+                                             ha_interface=False)
+
+        r1_chsfr = mock.patch.object(self.agent,
+                                     'check_ha_state_for_router').start()
+        r2_chsfr = mock.patch.object(self.failover_agent,
+                                     'check_ha_state_for_router').start()
+
+        utils.wait_until_true(lambda: router1.ha_state == 'master')
+
+        self.agent._process_updated_router(router1.router)
+        self.assertTrue(r1_chsfr.called)
+        self.failover_agent._process_updated_router(router2.router)
+        self.assertFalse(r2_chsfr.called)
+
     def _setup_dvr_router_static_routes(
         self, router_namespace=True, check_fpr_int_rule_delete=False):
         """Test to validate the extra routes on dvr routers."""
